using AutoScan;
using AutoScan.Interfaces;
using CasaBotApp.TelegramBot;
using ControlServer;
using Microsoft.Extensions.Logging;
using System.Diagnostics;
using Telegram.Bots.Types;
using BotCommand = CasaBotApp.TelegramBot.BotCommand;

namespace CasaBotApp.Extensions;

public class AlarmBotOrquestrator
{
    private readonly ILogger<AlarmBotOrquestrator> _logger;
    private readonly BotHandler _botHandler;
    private readonly AutoScanApp _autoScanApp;
    private readonly IControlServer _controlServer;
    private readonly IShinobiLinkFactory _shinobiLinkFactory;
    private readonly IDVRConnector _dvrConnector;

    public AlarmBotOrquestrator(ILogger<AlarmBotOrquestrator> logger, BotHandler botHandler, AutoScanApp autoScanApp, IControlServer controlServer, IShinobiLinkFactory shinobiLinkFactory, IDVRConnector dvrConnector)
    {
        _logger = logger;
        _botHandler = botHandler;
        _autoScanApp = autoScanApp;
        _controlServer = controlServer;
        _shinobiLinkFactory = shinobiLinkFactory;
        _dvrConnector = dvrConnector;
    }

    public void RegisterCommands()
    {
        _botHandler.RegisterCommand(new BotCommand
        {
            Command = "/soyandre",
            Description = "Soy Andre",
            Action = async (message, ctx) =>
            {
                await ctx.Responder(message, "Hola vida, te amo mucho ❤️");
            }
        });
        _botHandler.RegisterCommand(new BotCommand
        {
            Command = "/startScan",
            Description = "Start a scan of last night images",
            Action = async (message, ctx) =>
            {
                await ctx.Responder(message, "Starting scan 🔍📼");
                await _autoScanApp.StartNewScan();
            }
        });
        _botHandler.RegisterCommand(new BotCommand
        {
            Command = "/lastscan",
            Description = "Send the images from the last scan",
            Action = async (message, ctx) =>
            {
                var images = _autoScanApp.GetLastScanPictures();
                if (images.Length == 0)
                {
                    await ctx.Responder(message, "No images found");
                    return;
                }
                await _botHandler.SendPhotos(message.Chat.Id, images);
            }
        });
        
        _botHandler.RegisterCommand(new BotCommand()
        {
            Command = "/now",
            Description = "Send the current snapshot",
            Action = async (msg, ctx) =>
            {
                var stopwatch = Stopwatch.StartNew();
                stopwatch.Start();
                var outputPath = await TakeSnapshot();
                stopwatch.Stop();
                if (string.IsNullOrEmpty(outputPath))
                {
                    await ctx.Responder(msg, "Error taking snapshot");
                    return;
                }
                _ = _botHandler.SendPhoto(msg.Chat.Id, outputPath, "Current snapshot");
                _ = _botHandler.SendText(msg.Chat.Id, $"It took {stopwatch.ElapsedMilliseconds} ms to take the picture");
            }
        });
        _botHandler.RegisterCommand(new BotCommand()
        {
            Command = "/disarm",
            Description = "Disarm the Door Sensor",
            Action = async (msg, ctx) =>
            {
                await ctx.Responder(msg, "Disarming the door sensor");
                _controlServer.RequestDisarm();
            }
        });
        
        _botHandler.OnReply = async msg =>
        {
            var originalMsg = msg.ReplyToMessage;

            // Check if the original message is a photo and has a caption
            if (originalMsg is not PhotoMessage photoMessage || photoMessage.Caption is null)
                return;
            
            var videoPath = _autoScanApp.GetVideoPath(photoMessage.Caption);
            if (string.IsNullOrEmpty(videoPath))
            {
                await _botHandler.SendText(msg.Chat.Id, "No video found for this image");
                return;
            }
            
            await _botHandler.SendVideo(msg.Chat.Id, videoPath);
        };
        
    }
    
    public void RegisterAutoScanApp()
    {
        _autoScanApp.OnScanCompleted = async () =>
        {
            _logger.LogInformation("Scan completed at {At}", DateTime.Now);
            try
            {
                var images = _autoScanApp.GetLastScanPictures();
                if (images.Length == 0)
                {
                    await _botHandler.UpdateText("No images found");
                    return;
                }
                await _botHandler.UpdateText($"Scan completed, found {images.Length} images");
                await _botHandler.UpdatePhotos(images);    
            }catch(Exception ex)
            {
                _logger.LogError(ex, "Error while sending message");
            }
        };
    }
    
    public void RegisterControlServer()
    {
        _controlServer.OnEvent(async se =>
        {
            var mediaPath = await TakeSnapshot();
            if (string.IsNullOrEmpty(mediaPath))
            {
                await _botHandler.AlertText("Unauthorized access detected 🚨🚨🚨, but no media available");
                return;
            }
            if (se.Type == EventType.Fired)
            {
                await _botHandler.AlertPhoto(mediaPath,
                    "Unauthorized access detected 🚨 🚨 🚨",
                    [
                        new(OptionType.Url, "Camera Feed", _shinobiLinkFactory.BuildFeedLink()),
                        new(OptionType.Action, "Authorize", $"authorize-{se.EventId}", (data, chatId ) =>
                        {
                            _logger.LogWarning("Authorizing event {EventId}", se.EventId);
                            _controlServer.AuthorizeEvent(se.EventId);
                            return Task.FromResult("Authorization not implemented");
                        }),
                    ]);
            }

            if (se.Type == EventType.DisarmedEntrance)
            {
                await _botHandler.UpdateText("Authorize access");
            }
        });
    }
    private async Task<string> TakeSnapshot()
    {
        var timer = new Stopwatch();
        timer.Start();
        var outputPath = Path.Combine(".", "media", "snp", "something.jpeg");
        var originalFeed = await _dvrConnector.GetVideoStream();
        var ffmArgs = $"-y -i \"{originalFeed}\" -ss 00:00:00.500 -vframes 1 {outputPath}";
        
        var process = new Process
        {
            StartInfo = new ProcessStartInfo
            {
                //To change this, I need to make sure ffmpeg is installed
                FileName = "./dvr-scanner/ffmpeg.exe",
                Arguments = ffmArgs,
                RedirectStandardOutput = true,
                RedirectStandardError = true,
                UseShellExecute = false,
                CreateNoWindow = true,
            }
        };
        process.Start();
        await process.WaitForExitAsync();
        // You can read the output here.
        // var output = await process.StandardOutput.ReadToEndAsync();
        // var error = await process.StandardError.ReadToEndAsync();
        timer.Stop();
        _logger.LogDebug("Taking snapshot took {Elapsed} ms", timer.ElapsedMilliseconds);
        
        if(process.ExitCode != 0)
        {
            _logger.LogError("Error taking snapshot, exit code: {ExitCode}", process.ExitCode);
            return string.Empty;
        }
        return outputPath;
    }
}